@@ -8,8 +8,10 @@ open Indent
 
 open Atd.Ast
 open Mapping
+module Json = Atd.Json
 
 let target : Ocaml.target = Json
+let annot_schema = Ocaml.annot_schema_of_target target
 
 (*
   OCaml code generator (json readers and writers)
@@ -57,7 +59,7 @@ let make_ocaml_json_intf ~with_create buf deref defs =
     bprintf buf "\
 val write_%s :%s
   Bi_outbuf.t -> %s -> unit
-  (** Output a JSON value of type {!%s}. *)
+  (** Output a JSON value of type {!type:%s}. *)
 
 "
       s writer_params
@@ -67,7 +69,7 @@ val write_%s :%s
     bprintf buf "\
 val string_of_%s :%s
   ?len:int -> %s -> string
-  (** Serialize a value of type {!%s}
+  (** Serialize a value of type {!type:%s}
       into a JSON string.
       @param len specifies the initial length
                  of the buffer used internally.
@@ -81,7 +83,7 @@ val string_of_%s :%s
     bprintf buf "\
 val read_%s :%s
   Yojson.Safe.lexer_state -> Lexing.lexbuf -> %s
-  (** Input JSON data of type {!%s}. *)
+  (** Input JSON data of type {!type:%s}. *)
 
 "
       s reader_params
@@ -91,7 +93,7 @@ val read_%s :%s
     bprintf buf "\
 val %s_of_string :%s
   string -> %s
-  (** Deserialize JSON data of type {!%s}. *)
+  (** Deserialize JSON data of type {!type:%s}. *)
 
 "
       s reader_params
@@ -151,6 +153,9 @@ let rec get_writer_name
   | String (_, String, String) ->
       "Yojson.Safe.write_string"
 
+  | Abstract (_, Abstract, Abstract) ->
+      "Yojson.Safe.write_json"
+
   | Tvar (_, s) -> "write_" ^ (Ox_emit.name_of_var s)
 
   | Name (_, s, args, None, None) ->
@@ -168,7 +173,9 @@ let rec get_writer_name
       if paren && l <> [] then "(" ^ s ^ ")"
       else s
 
-  | _ -> assert false
+  | Unit _ | Bool _ | Int _ | Float _ | String _ | Abstract _
+  | Sum _ | Record _ | Tuple _ | List _ | Option _ | Nullable _
+  | Wrap _ | Name _ | External _ -> assert false
 
 
 let get_left_writer_name p name param =
@@ -202,6 +209,8 @@ let rec get_reader_name
 
   | String (_, String, String) -> "Atdgen_runtime.Oj_run.read_string"
 
+  | Abstract (_, Abstract, Abstract) -> "Atdgen_runtime.Oj_run.read_json"
+
   | Tvar (_, s) -> "read_" ^ Ox_emit.name_of_var s
 
   | Name (_, s, args, None, None) ->
@@ -219,7 +228,9 @@ let rec get_reader_name
       if paren && l <> [] then "(" ^ s ^ ")"
       else s
 
-  | _ -> assert false
+  | Unit _ | Bool _ | Int _ | Float _ | String _ | Abstract _
+  | Sum _ | Record _ | Tuple _ | List _ | Option _ | Nullable _
+  | Wrap _ | Name _ | External _ -> assert false
 
 
 let get_left_reader_name p name param =
@@ -234,9 +245,7 @@ let get_left_of_string_name p name param =
 let write_with_adapter adapter writer =
   match adapter.Json.ocaml_adapter with
   | None -> writer
-  | Some adapter_path ->
-      let restore =
-        Oj_mapping.json_restorer_of_adapter_path adapter_path in
+  | Some { restore; _ } ->
       [
         Line (
           sprintf "Atdgen_runtime.Oj_run.write_with_adapter %s (" restore
@@ -258,6 +267,7 @@ let rec make_writer ?type_constraint p (x : Oj_mapping.t) : Indent.t list =
   | Int _
   | Float _
   | String _
+  | Abstract _
   | Name _
   | External _
   | Tvar _ -> [ Line (get_writer_name p x) ]
@@ -386,7 +396,8 @@ let rec make_writer ?type_constraint p (x : Oj_mapping.t) : Indent.t list =
            ]
       )
 
-  | _ -> assert false
+  | Sum _ | Record _ | Tuple _ | List _ | Option _ | Nullable _ | Wrap _ ->
+      assert false
 
 
 and make_variant_writer p ~tick ~open_enum x : Indent.t list =
@@ -583,9 +594,7 @@ let study_record ~ocaml_version fields =
 let read_with_adapter adapter reader =
   match adapter.Json.ocaml_adapter with
   | None -> reader
-  | Some adapter_path ->
-      let normalize =
-        Oj_mapping.json_normalizer_of_adapter_path adapter_path in
+  | Some { normalize; _ } ->
       [
         Line (
           sprintf "Atdgen_runtime.Oj_run.read_with_adapter %s (" normalize
@@ -601,6 +610,7 @@ let rec make_reader p type_annot (x : Oj_mapping.t) : Indent.t list =
   | Int _
   | Float _
   | String _
+  | Abstract _
   | Name _
   | External _
   | Tvar _ -> [ Line (get_reader_name p x) ]
@@ -770,7 +780,8 @@ let rec make_reader p type_annot (x : Oj_mapping.t) : Indent.t list =
            ]
       )
 
-  | _ -> assert false
+  | Sum _ | Record _ | Tuple _ | List _ | Option _ | Nullable _ | Wrap _ ->
+      assert false
 
 (*
    Return a pair (optional json constructor, expression) to be converted
@@ -1323,11 +1334,13 @@ let make_ocaml_files
     match atd_file with
       Some file ->
         Atd.Util.load_file
+          ~annot_schema
           ~expand:false ~inherit_fields:true ~inherit_variants:true
           ?pos_fname ?pos_lnum
           file
     | None ->
         Atd.Util.read_channel
+          ~annot_schema
           ~expand:false ~inherit_fields:true ~inherit_variants:true
           ?pos_fname ?pos_lnum
           stdin
@@ -1359,7 +1372,7 @@ let make_ocaml_files
     in
     sprintf "\
 (* Auto-generated from %s *)
-[@@@ocaml.warning \"-27-32-35-39\"]" src
+[@@@ocaml.warning \"-27-32-33-35-39\"]" src
   in
   let mli =
     make_mli ~header ~opens ~with_typedefs ~with_create ~with_fundefs
