@@ -209,6 +209,10 @@ let test_biniou_missing_cell () =
     Test.extended_tuple_of_string
     (Test.string_of_base_tuple test_missing_tuple)
 
+let test_biniou_read_write_unit () =
+  section "biniou read/write unit";
+  Test_unit_biniou_b.t_of_string (Test_unit_biniou_b.string_of_t ())
+
 let test_json_missing_field () =
   section "json missing record fields";
   expect_error
@@ -468,16 +472,6 @@ let test_json_streams () =
   in
   check (l = l'')
 
-let test_raw_json () =
-  section "raw json";
-  let x = { Test3j_t.foo = 12345;
-            bar = `List [ `Int 12; `String "abc" ];
-            baz = `Bool false }
-  in
-  let s = Test3j_j.string_of_t x in
-  let x' = Test3j_j.t_of_string s in
-  check (x = x')
-
 let test_wrapping_ints () =
   section "ocaml wrapping - ints";
   let x = Test_lib.Natural.wrap 7 in
@@ -563,6 +557,29 @@ let test_adapted () =
   check (expected_a = rewritten_a);
   check (expected_b = rewritten_b)
 
+let test_adapted_f () =
+  section "read and write a variant represented as a json object \
+           with a `type` field (pair of functions)";
+  let json_a = {| {
+    "type": "fa",
+    "thing": "abc",
+    "other_thing": true,
+    "ignored": 555
+  } |} in
+  let json_b =  {| {
+    "thing": 123,
+    "type": "fb",
+    "other_thing": true
+  } |} in
+  let expected_a = {|{"type":"fa","thing":"abc","other_thing":true}|} in
+  let expected_b = {|{"type":"fb","thing":123}|} in
+  let a = Test3j_j.adapted_f_of_string json_a in
+  let b = Test3j_j.adapted_f_of_string json_b in
+  let rewritten_a = Test3j_j.string_of_adapted_f a in
+  let rewritten_b = Test3j_j.string_of_adapted_f b in
+  check (expected_a = rewritten_a);
+  check (expected_b = rewritten_b)
+
 let test_one_field () =
   section "test variants represented with single-field json objects";
   let a_json = {| {"a": true} |} in
@@ -630,6 +647,42 @@ let test_polymorphic_wrap () =
     Test_polymorphic_wrap_j.t_of_string Yojson.Safe.read_string json_out in
   check (x = x2)
 
+let test_encoding_int64 () =
+  section "json encoding int64 as string";
+  let encoded = Test_int64_enc_j.string_of_int64 Int64.max_int in
+  check (String.equal encoded {|"9223372036854775807"|})
+
+let test_encoding_decoding_int64 () =
+  section "json encoding & decoding int64";
+  let encoded = Test_int64_enc_j.string_of_int64 Int64.max_int in
+  let decoded = Test_int64_enc_j.int64_of_string encoded in
+  check (decoded = Int64.max_int)
+
+let test_raw_json () =
+  section "raw json";
+  let x = { Test3j_t.foo = 12345;
+            bar = `List [ `Int 12; `String "abc" ];
+            baz = `Bool false }
+  in
+  let s = Test3j_j.string_of_t x in
+  let x' = Test3j_j.t_of_string s in
+  check (x = x')
+
+let test_abstract_types () =
+  section "abstract types";
+  let input = ["a", 1; "b", 2] in
+  let encoded = Test_abstract_j.string_of_int_assoc_list input in
+  let decoded = Test_abstract_j.int_assoc_list_of_string encoded in
+  check (decoded = input)
+
+(* see also 'test_raw_json' which uses the legacy, more complicated way *)
+let test_untyped_json () =
+  section "untyped json";
+  let input = {|["hello",42,{}]|} in
+  let decoded = Test_abstract_j.any_items_of_string input in
+  let encoded = Test_abstract_j.string_of_any_items decoded in
+  check (encoded = input)
+
 let all_tests = [
   test_ocaml_internals;
   test_biniou_missing_field;
@@ -640,6 +693,7 @@ let all_tests = [
   test_json_assoc_list;
   test_json_assoc_array;
   test_json_int_ocaml_float;
+  test_biniou_read_write_unit;
   test_biniou_correctness;
   test_json_correctness;
   test_json_space;
@@ -659,11 +713,16 @@ let all_tests = [
   test_json_float_decimals;
   test_patch;
   test_adapted;
+  test_adapted_f;
   test_one_field;
   test_tag_field_emulation;
   test_tag_field_emulation_with_catchall;
   test_json_open_enum;
   test_ambiguous_record;
+  test_encoding_int64;
+  test_encoding_decoding_int64;
+  test_abstract_types;
+  test_untyped_json;
 ]
 
 (* TODO: use Alcotest to run the test suite. *)
